/*
  Copyright (C) 1998-1999 Johan Redestig

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "stdafx.h"
#include "MenuItem.h"
#include "Painter.h"
#include "PopupMenu.h"

BOOL MenuItem::m_bDrawBevel = TRUE;
DWORD MenuItem::m_nDarkColor = 0x00000000;
DWORD MenuItem::m_nLightColor = 0x00FFFFFF;
int MenuItem::m_nIndent = 0;
BOOL MenuItem::m_bAlignAllText = FALSE;

MenuItem::MenuItem()
{
	m_pParent = NULL;
	m_nShortcut = -1; // not a key code
	m_nWidth = 110; // todo: make dynamic
	m_nHeight = 18; // todo: make dynamic

	m_pForeground = NULL;
	m_pBackground = NULL;

	m_pActiveForeground = NULL;
	m_pActiveBackground = NULL;

	m_bActive = FALSE;
	m_hIcon = NULL;

	m_nSortPriority = 0;
}

MenuItem::~MenuItem()
{
	if(m_hIcon != NULL)
	{
		DeleteObject(m_hIcon);
		m_hIcon = NULL;
	}
}

// Paint the menu item att the dc.
// nState ::= SELECTED | DEFAULT
void MenuItem::Paint(HDC hDC)
{
	SetBkMode(hDC, TRANSPARENT); // todo: move to a painter...

	// Paint the background
	if(m_bActive && m_pActiveBackground)
		m_pActiveBackground->Paint(this, hDC);
	else if(m_pBackground)
		m_pBackground->Paint(this, hDC);

	// Paint the foreground ...

	if(m_hIcon)
		//DrawIcon(hDC, 3, m_nTop + 1, m_hIcon);
		DrawIconEx(hDC, 3 + GetIndent(), m_nTop, m_hIcon, GetHeight()-3, GetHeight()-3, 0, NULL, DI_NORMAL);

}

// Execute the command that is associated with the menu item
void MenuItem::Invoke()
{
	// default implementation is a nop
	return;
}

// A key has been pressed
BOOL MenuItem::Key(int nKey)
{
	if(m_nShortcut == nKey)
		Invoke();
	return FALSE;
}

// Mouse command MOUSEMOVE, LB_PRESSED, ...
void MenuItem::Mouse(int nMsg, int x, int y)
{
	RECT r;
	POINT p;
	p.x = x;
	p.y = y;
	GetItemRect(&r);
	
	if(PtInRect(&r, p))
	{
		switch(nMsg)
		{
		case WM_LBUTTONUP:
			Invoke();
			break;
		case WM_MOUSEMOVE:
			Active(TRUE);
			break;
		default:
			break;
		}
	}
	else
		Active(FALSE);
}

void MenuItem::PaintBevel(HDC hDC, int nTop)
{
/*	RECT r;

	r.top = nTop;
	r.left = 0;
	r.right = r.left + GetWidth();
	r.bottom = r.top + Height();

	DrawEdge(hDC, &r, EDGE_ETCHED, BF_ADJUST | BF_RECT);*/
	HPEN hPen;
	HPEN hOldPen;
	
	hPen = CreatePen(PS_SOLID, 1, m_nLightColor);
	hOldPen = (HPEN) SelectObject(hDC, hPen);

	// First we draw the white frame on the left/top
	MoveToEx(hDC, 0, nTop, NULL);
	LineTo(hDC, GetWidth(), nTop);
	MoveToEx(hDC, 0, nTop, NULL);
	LineTo(hDC, 0, nTop+GetHeight()-1);
	SelectObject(hDC, hOldPen);
	DeleteObject(hPen);

	// Now the black frame on the bottom/right
	hPen = CreatePen(PS_SOLID, 1, m_nDarkColor);
	hOldPen = (HPEN) SelectObject(hDC, hPen);
	MoveToEx(hDC, GetWidth()-1, nTop, NULL);
	LineTo(hDC, GetWidth()-1, nTop+GetHeight()-1);
	MoveToEx(hDC, 0, nTop + GetHeight() - 1, NULL);
	LineTo(hDC, GetWidth()-1, nTop+GetHeight()-1);

	SelectObject(hDC, hOldPen);
	DeleteObject(hPen);
}

void MenuItem::Timer(int nTimer)
{
	// execute timer messages
}

int MenuItem::IsOver(int x, int y)
{
	POINT p;
	RECT r;

	p.x = x;
	p.y = y;

	r.top = m_nTop;
	r.left = 0;
	r.bottom = r.top + GetHeight();
	r.right = r.left + GetWidth();

	return PtInRect(&r, p); // TODO: this is wrong ....fix it -1 .. 0 .. 1
}

void MenuItem::Attached(PopupMenu* pMenu)
{
	m_pParent = pMenu;
}

BOOL MenuItem::Active(BOOL bActive)
{
	RECT r;

	if(m_bActive != bActive)
	{
		// only redraw the window if the state changes
		m_bActive = bActive;

		GetItemRect(&r);

		// todo: no need to repaint entire window ...
//		InvalidateRect(m_pParent->GetWindow(),&r,TRUE);
//		UpdateWindow(m_pParent->GetWindow());	
		RedrawWindow(GetWindow(), &r, NULL, RDW_INVALIDATE);
	}

	return m_bActive;
}

void MenuItem::GetItemRect(LPRECT lpRect)
{
	lpRect->top = m_nTop;
	lpRect->left = 0;
	lpRect->right = lpRect->left + GetWidth();
	lpRect->bottom = lpRect->top + GetHeight();
}

void MenuItem::SetPainter(Painter* pBackground)
{
	m_pBackground = pBackground;
}

void MenuItem::SetActivePainter(Painter* pBackground)
{
	m_pActiveBackground = pBackground;
}

HWND MenuItem::GetWindow()
{
	if(m_pParent)
		return m_pParent->GetWindow();
	else
		return NULL;
}

void MenuItem::GetRegion(HRGN hRgn)
{
	m_pBackground->GetRegion(hRgn, this);
}

LRESULT MenuItem::Command(WPARAM wParam, LPARAM lParam)
{
	return 0;
}

// returns true iif m1 < m2
bool MenuItem::Compare(MenuItem* m1,MenuItem* m2)
{
	int m1_sort = m1->GetSortPriority();
	int m2_sort = m2->GetSortPriority();

	if(m1_sort == m2_sort)
		return lstrcmpi(m1->GetSortString(), m2->GetSortString()) < 0;
	else
		return m1_sort > m2_sort;
}

